/**
 * @internal
 */
export declare class RawSha256 {
    private state;
    private temp;
    private buffer;
    private bufferLength;
    private bytesHashed;
    /**
     * @internal
     */
    finished: boolean;
    update(data: Uint8Array): void;
    digest(): Uint8Array;
    private hashBuffer;
}
