/**
 * @internal
 */
export declare const BLOCK_SIZE: number;
/**
 * @internal
 */
export declare const DIGEST_LENGTH: number;
/**
 * @internal
 */
export declare const KEY: Uint32Array;
/**
 * @internal
 */
export declare const INIT: number[];
/**
 * @internal
 */
export declare const MAX_HASHABLE_LENGTH: number;
