export declare const hashTestVectors: Array<[Uint8Array, Uint8Array]>;
/**
 * @see https://tools.ietf.org/html/rfc4231
 */
export declare const hmacTestVectors: Array<[Uint8Array, Uint8Array, Uint8Array]>;
